
from .AbstractSelector import AbstractSelector


class InputText(AbstractSelector):
    """
    Just a selector that finds an input text by its name.
    """
    def __init__(self, name=None):
        super(AbstractSelector, self).__init__()

        xpath_selectors = ["//input[@type='text']",
                           "//input[not(@type)]"]

        if name:
            for i in range(len(xpath_selectors)):
                xpath_selectors[i] += "[@name='%s']" % name

        self._xpath_selectors = xpath_selectors

    def get_selectors(self):
        # CSS selector.
        return self._xpath_selectors



from .AbstractSelector import AbstractSelector


class XPath(AbstractSelector):
    """
    Just a selector that finds some XPath.
    """
    def __init__(self, selector=None):
        super(AbstractSelector, self).__init__()

        if not selector.startswith('//'):
            selector = "xpath:" + selector

        self._selector = selector

    def get_selectors(self):
        """ Return the XPath selector itself """
        return [self._selector]




from .GermaniumDriver import GermaniumDriver
from .iframe import iframe

from .util import get_attributes
from .util import click, right_click, double_click, hover
from .util import type_keys
from .util import wait



from .AbstractSelector import AbstractSelector


class Element(AbstractSelector):
    def __init__(self,
                 tag_name=None,
                 index=-1,
                 exact_text=None,
                 contains_text=None,
                 css_classes=[],
                 exact_attributes={},
                 contains_attributes={}):
        """ A general element selector. """
        super(Element, self).__init__()

        if exact_text and contains_text:
            raise Exception("Having the exact text to be matched, "
                            "and a partial text to be searched is not supported.")

        xpath_locator = '//' + tag_name

        if contains_text:
            xpath_locator += "[contains(normalize-space(string()), '%s')]" % contains_text

        if exact_text:
            xpath_locator += "[string() = '%s']" % exact_text

        for css_class in css_classes:
            xpath_locator += '[contains(concat(" ", @class, " "), " %s ")]' % css_class

        for k, v in exact_attributes.items():
            xpath_locator += '[@%s = "%s"]' % (k, v)

        for k, v in contains_attributes.items():
            xpath_locator += "[contains(normalize-space(@%s), '%s')]" % (k, v)

        if index >= 0:
            xpath_locator = '(%s)[%d]' % (xpath_locator, index)

        self._xpath_locator = xpath_locator

    def get_selectors(self):
        return [self._xpath_locator]



from .AbstractSelector import AbstractSelector, PositionalFilterSelector

class TableRow(AbstractSelector):
    def __init__(self):
        super(TableRow, self).__init__()

        self._cell_selectors = []

    def containing(self, cell_selector):
        self._cell_selectors.append(cell_selector)
        return self

    def get_selectors(self):
        result = ["//tr"]

        for cell_selector in self._cell_selectors:
            if isinstance(cell_selector, PositionalFilterSelector):
                raise Exception("Positional selectors are not supported in TableRow.containing(), only "
                                "selectors that will return XPath global searches, without positional "
                                "references.")

            string_selectors = cell_selector.get_selectors()

            for string_selector in string_selectors:
                if not string_selector.startswith("//") \
                        and not string_selector.startswith("xpath://"):
                    raise Exception("Only selectors that return XPath are supported for `TableRow.containing` "
                                    "calls. Currently `%s` was found. In case you want to use a CSS selector, "
                                    "use instead the `germanium.selectors.Element` selector, since it "
                                    "generates XPath, and will function correctly with TableRow selectors." % string_selector)

            if len(string_selectors) > 1:
                new_result = []

                for string_selector in string_selectors:
                    if string_selector.startswith("xpath://"):
                        string_selector = string_selector[6:]

                    for current_result in result:
                        new_result.append("%s[.%s]" % (current_result, string_selector))

                result = new_result
            else:
                for i in range(len(result)):
                    string_selector = string_selectors[0]

                    if string_selector.startswith("xpath://"):
                        string_selector = string_selector[6:]

                    result[i] += "[.%s]" % string_selector

        return result



from germanium.util import double_click as double_click_impl
from .global_germanium_instance import *


def double_click(selector):
    """
    Double click the element with the given selector.
    """
    if not get_instance():
        raise Exception("You need to start a browser first with open_browser()")

    double_click_impl(get_instance(), selector)




from .AbstractSelector import AbstractSelector


class CheckBox(AbstractSelector):
    """
    Just a selector that finds a checkbox input by its name.
    """
    def __init__(self, name=None):
        super(CheckBox, self).__init__()
        self._input_name = name

    def get_selectors(self):
        # CSS selector.
        xpath_selector = "//input[@type='checkbox']"

        if self._input_name:
            xpath_selector += "[@name='%s']" % self._input_name

        return [xpath_selector]
